@@ -48,7 +48,7 @@ data_offer_accept (struct wl_client *client,
    * this be a wl_data_device request? */
 
   if (offer->source)
-    offer->source->accept (offer->source, serial, mime_type);
+    wl_data_source_send_target (offer->source->resource, mime_type);
 }
 
 static void
@@ -58,9 +58,9 @@ data_offer_receive (struct wl_client *client, struct wl_resource *resource,
   MetaWaylandDataOffer *offer = wl_resource_get_user_data (resource);
 
   if (offer->source)
-    offer->source->send (offer->source, mime_type, fd);
-  else
-    close (fd);
+    wl_data_source_send_send (offer->source->resource, mime_type, fd);
+
+  close (fd);
 }
 
 static void
@@ -82,7 +82,8 @@ destroy_data_offer (struct wl_resource *resource)
 
   if (offer->source)
     wl_list_remove (&offer->source_destroy_listener.link);
-  free (offer);
+
+  g_slice_free (MetaWaylandDataOffer, offer);
 }
 
 static void
@@ -99,13 +100,9 @@ static struct wl_resource *
 meta_wayland_data_source_send_offer (MetaWaylandDataSource *source,
                                      struct wl_resource *target)
 {
-  MetaWaylandDataOffer *offer;
+  MetaWaylandDataOffer *offer = g_slice_new0 (MetaWaylandDataOffer);
   char **p;
 
-  offer = malloc (sizeof *offer);
-  if (offer == NULL)
-    return NULL;
-
   offer->source = source;
   offer->source_destroy_listener.notify = destroy_offer_data_source;
 
@@ -178,8 +175,7 @@ destroy_drag_focus (struct wl_listener *listener, void *data)
 
 static void
 drag_grab_focus (MetaWaylandPointerGrab *grab,
-                 MetaWaylandSurface     *surface,
-		 const ClutterEvent     *event)
+                 MetaWaylandSurface     *surface)
 {
   MetaWaylandDragGrab *drag_grab = (MetaWaylandDragGrab*) grab;
   MetaWaylandSeat *seat = drag_grab->seat;
@@ -260,7 +256,7 @@ data_device_end_drag_grab (MetaWaylandDragGrab *drag_grab)
   if (drag_grab->drag_data_source)
     wl_list_remove (&drag_grab->drag_data_source_listener.link);
 
-  drag_grab_focus (&drag_grab->generic, NULL, NULL);
+  drag_grab_focus (&drag_grab->generic, NULL);
 
   meta_wayland_pointer_end_grab (drag_grab->generic.pointer);
   g_slice_free (MetaWaylandDragGrab, drag_grab);
@@ -318,8 +314,12 @@ data_device_start_drag (struct wl_client *client,
 {
   MetaWaylandSeat *seat = wl_resource_get_user_data (resource);
   MetaWaylandDragGrab *drag_grab;
-  /* FIXME: Check that client has implicit grab on the origin
-   * surface that matches the given time. */
+
+  if ((seat->pointer.button_count == 0 ||
+       seat->pointer.grab_serial != serial ||
+       !seat->pointer.focus_surface ||
+       seat->pointer.focus_surface != wl_resource_get_user_data (origin_resource)))
+    return;
 
   /* FIXME: Check that the data source type array isn't empty. */
 
@@ -332,6 +332,7 @@ data_device_start_drag (struct wl_client *client,
   drag_grab->generic.pointer = &seat->pointer;
 
   drag_grab->drag_client = client;
+  drag_grab->seat = seat;
 
   if (source_resource)
     {
@@ -375,7 +376,7 @@ destroy_selection_data_source (struct wl_listener *listener, void *data)
     }
 }
 
-void
+static void
 meta_wayland_seat_set_selection (MetaWaylandSeat *seat,
                                  MetaWaylandDataSource *source,
                                  guint32 serial)
@@ -389,7 +390,7 @@ meta_wayland_seat_set_selection (MetaWaylandSeat *seat,
 
   if (seat->selection_data_source)
     {
-      seat->selection_data_source->cancel (seat->selection_data_source);
+      wl_data_source_send_cancelled (seat->selection_data_source->resource);
       wl_list_remove (&seat->selection_data_source_listener.link);
       seat->selection_data_source = NULL;
     }
@@ -449,47 +450,21 @@ static const struct wl_data_device_interface data_device_interface = {
 static void
 destroy_data_source (struct wl_resource *resource)
 {
-  MetaWaylandDataSource *source = wl_container_of (resource, source, resource);
+  MetaWaylandDataSource *source = wl_resource_get_user_data (resource);
   char **p;
 
   wl_array_for_each (p, &source->mime_types) free (*p);
 
   wl_array_release (&source->mime_types);
-}
 
-static void
-client_source_accept (MetaWaylandDataSource *source,
-                      guint32 time, const char *mime_type)
-{
-  wl_data_source_send_target (source->resource, mime_type);
-}
-
-static void
-client_source_send (MetaWaylandDataSource *source,
-                    const char *mime_type, int32_t fd)
-{
-  wl_data_source_send_send (source->resource, mime_type, fd);
-  close (fd);
-}
-
-static void
-client_source_cancel (MetaWaylandDataSource *source)
-{
-  wl_data_source_send_cancelled (source->resource);
+  g_slice_free (MetaWaylandDataSource, source);
 }
 
 static void
 create_data_source (struct wl_client *client,
                     struct wl_resource *resource, guint32 id)
 {
-  MetaWaylandDataSource *source;
-
-  source = malloc (sizeof *source);
-  if (source == NULL)
-    {
-      wl_resource_post_no_memory (resource);
-      return;
-    }
+  MetaWaylandDataSource *source = g_slice_new0 (MetaWaylandDataSource);
 
   source->resource = wl_resource_create (client, &wl_data_source_interface,
 					 MIN (META_WL_DATA_SOURCE_VERSION,
@@ -497,10 +472,6 @@ create_data_source (struct wl_client *client,
   wl_resource_set_implementation (source->resource, &data_source_interface,
 				  source, destroy_data_source);
 
-  source->accept = client_source_accept;
-  source->send = client_source_send;
-  source->cancel = client_source_cancel;
-
   wl_array_init (&source->mime_types);
 }
 
