@@ -138,23 +138,11 @@ meta_wayland_keyboard_take_keymap (MetaWaylandKeyboard *keyboard,
       return;
     }
 
-  if (xkb_info->keymap)
-    xkb_keymap_unref (xkb_info->keymap);
+  xkb_keymap_unref (xkb_info->keymap);
   xkb_info->keymap = keymap;
 
-  xkb_info->shift_mod =
-    xkb_map_mod_get_index (xkb_info->keymap, XKB_MOD_NAME_SHIFT);
-  xkb_info->caps_mod =
-    xkb_map_mod_get_index (xkb_info->keymap, XKB_MOD_NAME_CAPS);
-  xkb_info->ctrl_mod =
-    xkb_map_mod_get_index (xkb_info->keymap, XKB_MOD_NAME_CTRL);
-  xkb_info->alt_mod =
-    xkb_map_mod_get_index (xkb_info->keymap, XKB_MOD_NAME_ALT);
-  xkb_info->mod2_mod = xkb_map_mod_get_index (xkb_info->keymap, "Mod2");
-  xkb_info->mod3_mod = xkb_map_mod_get_index (xkb_info->keymap, "Mod3");
-  xkb_info->super_mod =
-    xkb_map_mod_get_index (xkb_info->keymap, XKB_MOD_NAME_LOGO);
-  xkb_info->mod5_mod = xkb_map_mod_get_index (xkb_info->keymap, "Mod5");
+  xkb_state_unref (xkb_info->state);
+  xkb_info->state = xkb_state_new (keymap);
 
   keymap_str = xkb_map_get_as_string (xkb_info->keymap);
   if (keymap_str == NULL)
@@ -295,31 +283,6 @@ static const MetaWaylandKeyboardGrabInterface
   default_grab_modifiers,
 };
 
-static gboolean
-modal_key (MetaWaylandKeyboardGrab *grab,
-	   uint32_t                 time,
-	   uint32_t                 key,
-	   uint32_t                 state)
-{
-  /* FALSE means: let the event through to clutter */
-  return FALSE;
-}
-
-static void
-modal_modifiers (MetaWaylandKeyboardGrab *grab,
-		 uint32_t                 serial,
-		 uint32_t                 mods_depressed,
-		 uint32_t                 mods_latched,
-		 uint32_t                 mods_locked,
-		 uint32_t                 group)
-{
-}
-
-static MetaWaylandKeyboardGrabInterface modal_grab = {
-  modal_key,
-  modal_modifiers,
-};
-
 gboolean
 meta_wayland_keyboard_init (MetaWaylandKeyboard *keyboard,
                             struct wl_display   *display)
@@ -355,8 +318,8 @@ meta_wayland_keyboard_init (MetaWaylandKeyboard *keyboard,
 static void
 meta_wayland_xkb_info_destroy (MetaWaylandXkbInfo *xkb_info)
 {
-  if (xkb_info->keymap)
-    xkb_map_unref (xkb_info->keymap);
+  xkb_keymap_unref (xkb_info->keymap);
+  xkb_state_unref (xkb_info->state);
 
   if (xkb_info->keymap_area)
     munmap (xkb_info->keymap_area, xkb_info->keymap_size);
@@ -364,47 +327,7 @@ meta_wayland_xkb_info_destroy (MetaWaylandXkbInfo *xkb_info)
     close (xkb_info->keymap_fd);
 }
 
-static gboolean
-state_equal (MetaWaylandXkbState *one,
-	     MetaWaylandXkbState *two)
-{
-  return one->mods_depressed == two->mods_depressed &&
-    one->mods_latched == two->mods_latched &&
-    one->mods_locked == two->mods_locked &&
-    one->group == two->group;
-}
-
 static void
-set_modifiers (MetaWaylandKeyboard *keyboard,
-               guint32              serial,
-               ClutterEvent        *event)
-{
-  MetaWaylandKeyboardGrab *grab = keyboard->grab;
-  MetaWaylandXkbState new_state;
-  guint effective_state;
-
-  clutter_event_get_state_full (event,
-				NULL,
-				&new_state.mods_depressed,
-				&new_state.mods_latched,
-				&new_state.mods_locked,
-				&effective_state);
-  new_state.group = (effective_state >> 13) & 0x3;
-
-  if (state_equal (&keyboard->modifier_state, &new_state))
-    return;
-
-  keyboard->modifier_state = new_state;
-
-  grab->interface->modifiers (grab,
-                              serial,
-                              new_state.mods_depressed,
-			      new_state.mods_latched,
-			      new_state.mods_locked,
-                              new_state.group);
-}
-
-static gboolean
 update_pressed_keys (MetaWaylandKeyboard   *keyboard,
 		     uint32_t               evdev_code,
 		     gboolean               is_press)
@@ -415,13 +338,10 @@ update_pressed_keys (MetaWaylandKeyboard   *keyboard,
                                 keyboard->keys.size);
       uint32_t *k;
 
-      /* We want to ignore events that are sent because of auto-repeat. In
-	 the Clutter event stream these appear as a single key press
-	 event. We can detect that because the key will already have been
-	 pressed */
+      /* Make sure we don't already have this key. */
       for (k = keyboard->keys.data; k < end; k++)
         if (*k == evdev_code)
-          return TRUE;
+          return;
 
       /* Otherwise add the key to the list of pressed keys */
       k = wl_array_add (&keyboard->keys, sizeof (*k));
@@ -439,18 +359,44 @@ update_pressed_keys (MetaWaylandKeyboard   *keyboard,
           {
             *k = *(end - 1);
             keyboard->keys.size -= sizeof (*k);
-
-            goto found;
+            return;
           }
 
       g_warning ("unexpected key release event for key 0x%x", evdev_code);
-      return FALSE;
-
-    found:
-      (void) 0;
     }
+}
 
-  return FALSE;
+static guint
+evdev_code (const ClutterKeyEvent *event)
+{
+  /* clutter-xkb-utils.c adds a fixed offset of 8 to go into XKB's
+   * range, so we do the reverse here. */
+  return event->hardware_keycode - 8;
+}
+
+void
+meta_wayland_keyboard_update (MetaWaylandKeyboard *keyboard,
+                              const ClutterKeyEvent *event)
+{
+  MetaWaylandKeyboardGrab *grab = keyboard->grab;
+  gboolean is_press = event->type == CLUTTER_KEY_PRESS;
+  struct xkb_state *state = keyboard->xkb_info.state;
+  enum xkb_state_component changed_state;
+
+  update_pressed_keys (keyboard, evdev_code (event), is_press);
+
+  changed_state = xkb_state_update_key (state,
+                                        event->hardware_keycode,
+                                        is_press ? XKB_KEY_DOWN : XKB_KEY_UP);
+  if (changed_state == 0)
+    return;
+
+  grab->interface->modifiers (grab,
+                              wl_display_next_serial (keyboard->display),
+                              xkb_state_serialize_mods (state, XKB_STATE_MODS_DEPRESSED),
+                              xkb_state_serialize_mods (state, XKB_STATE_MODS_LATCHED),
+                              xkb_state_serialize_mods (state, XKB_STATE_MODS_LOCKED),
+                              xkb_state_serialize_layout (state, XKB_STATE_LAYOUT_EFFECTIVE));
 }
 
 gboolean
@@ -458,34 +404,20 @@ meta_wayland_keyboard_handle_event (MetaWaylandKeyboard *keyboard,
                                     const ClutterKeyEvent *event)
 {
   gboolean is_press = event->type == CLUTTER_KEY_PRESS;
-  guint xkb_keycode, evdev_code;
-  uint32_t serial;
-  gboolean autorepeat;
   gboolean handled;
 
-  xkb_keycode = event->hardware_keycode;
-  if (event->device == NULL ||
-      !clutter_input_device_keycode_to_evdev (event->device,
-					      xkb_keycode, &evdev_code))
-    evdev_code = xkb_keycode - 8; /* What everyone is doing in practice... */
-
-  autorepeat = update_pressed_keys (keyboard, evdev_code, is_press);
-
-  meta_verbose ("Handling key %s%s event code %d\n",
-		is_press ? "press" : "release",
-		autorepeat ? " (autorepeat)" : "",
-		xkb_keycode);
-
-  if (autorepeat)
+  /* Synthetic key events are for autorepeat. Ignore those, as
+   * autorepeat in Wayland is done on the client side. */
+  if (event->flags & CLUTTER_EVENT_FLAG_SYNTHETIC)
     return FALSE;
 
-  serial = wl_display_next_serial (keyboard->display);
-
-  set_modifiers (keyboard, serial, (ClutterEvent*)event);
+  meta_verbose ("Handling key %s event code %d\n",
+		is_press ? "press" : "release",
+		event->hardware_keycode);
 
   handled = keyboard->grab->interface->key (keyboard->grab,
 					    event->time,
-					    evdev_code,
+					    evdev_code (event),
 					    is_press);
 
   if (handled)
@@ -533,28 +465,16 @@ meta_wayland_keyboard_set_focus (MetaWaylandKeyboard *keyboard,
         {
           struct wl_client *client = wl_resource_get_client (keyboard->focus_resource);
           struct wl_display *display = wl_client_get_display (client);
+          struct xkb_state *state = keyboard->xkb_info.state;
           uint32_t serial = wl_display_next_serial (display);
 
-          /* If we're in a modal grab, the client is focused but doesn't see
-             modifiers or pressed keys (and fix that up when we exit the modal) */
-          if (keyboard->grab->interface == &modal_grab)
-            {
-              struct wl_array empty;
-              wl_array_init (&empty);
-
-              wl_keyboard_send_modifiers (keyboard->focus_resource, serial, 0, 0, 0, 0);
-              wl_keyboard_send_enter (keyboard->focus_resource, serial, keyboard->focus_surface->resource, &empty);
-            }
-          else
-            {
-              wl_keyboard_send_modifiers (keyboard->focus_resource, serial,
-                                          keyboard->modifier_state.mods_depressed,
-                                          keyboard->modifier_state.mods_latched,
-                                          keyboard->modifier_state.mods_locked,
-                                          keyboard->modifier_state.group);
-              wl_keyboard_send_enter (keyboard->focus_resource, serial, keyboard->focus_surface->resource,
-                                      &keyboard->keys);
-            }
+          wl_keyboard_send_modifiers (keyboard->focus_resource, serial,
+                                      xkb_state_serialize_mods (state, XKB_STATE_MODS_DEPRESSED),
+                                      xkb_state_serialize_mods (state, XKB_STATE_MODS_LATCHED),
+                                      xkb_state_serialize_mods (state, XKB_STATE_MODS_LOCKED),
+                                      xkb_state_serialize_layout (state, XKB_STATE_LAYOUT_EFFECTIVE));
+          wl_keyboard_send_enter (keyboard->focus_resource, serial, keyboard->focus_surface->resource,
+                                  &keyboard->keys);
 
           wl_resource_add_destroy_listener (keyboard->focus_resource, &keyboard->focus_resource_listener);
           keyboard->focus_serial = serial;
@@ -588,85 +508,6 @@ meta_wayland_keyboard_release (MetaWaylandKeyboard *keyboard)
   wl_array_release (&keyboard->keys);
 }
 
-gboolean
-meta_wayland_keyboard_begin_modal (MetaWaylandKeyboard *keyboard,
-				   guint32              timestamp)
-{
-  MetaWaylandKeyboardGrab *grab;
-  uint32_t *end = (void *) ((char *) keyboard->keys.data +
-			    keyboard->keys.size);
-  uint32_t *k;
-  uint32_t serial;
-
-  meta_verbose ("Asked to acquire modal keyboard grab, timestamp %d\n", timestamp);
-
-  if (keyboard->grab != &keyboard->default_grab)
-    return FALSE;
-
-  if (keyboard->focus_surface)
-    {
-      /* Fake key release events for the focused app */
-      serial = wl_display_next_serial (keyboard->display);
-      keyboard->grab->interface->modifiers (keyboard->grab,
-					    serial,
-					    0, 0, 0, 0);
-
-      for (k = keyboard->keys.data; k < end; k++)
-	{
-	  keyboard->grab->interface->key (keyboard->grab,
-					  timestamp,
-					  *k, 0);
-	}
-    }
-
-  grab = g_slice_new0 (MetaWaylandKeyboardGrab);
-  grab->interface = &modal_grab;
-  meta_wayland_keyboard_start_grab (keyboard, grab);
-
-  meta_verbose ("Acquired modal keyboard grab, timestamp %d\n", timestamp);
-
-  return TRUE;
-}
-
-void
-meta_wayland_keyboard_end_modal (MetaWaylandKeyboard *keyboard,
-				 guint32              timestamp)
-{
-  MetaWaylandKeyboardGrab *grab;
-  uint32_t *end = (void *) ((char *) keyboard->keys.data +
-			    keyboard->keys.size);
-  uint32_t *k;
-  uint32_t serial;
-
-  grab = keyboard->grab;
-
-  g_assert (grab->interface == &modal_grab);
-
-  meta_wayland_keyboard_end_grab (keyboard);
-  g_slice_free (MetaWaylandKeyboardGrab, grab);
-
-  if (keyboard->focus_surface)
-    {
-      /* Fake key press events for the focused app */
-      serial = wl_display_next_serial (keyboard->display);
-      keyboard->grab->interface->modifiers (keyboard->grab,
-					    serial,
-					    keyboard->modifier_state.mods_depressed,
-					    keyboard->modifier_state.mods_latched, 
-					    keyboard->modifier_state.mods_locked,
-					    keyboard->modifier_state.group);
-
-      for (k = keyboard->keys.data; k < end; k++)
-	{
-	  keyboard->grab->interface->key (keyboard->grab,
-					  timestamp,
-					  *k, 1);
-	}
-    }
-
-  meta_verbose ("Released modal keyboard grab, timestamp %d\n", timestamp);
-}
-
 void
 meta_wayland_keyboard_set_keymap_names (MetaWaylandKeyboard *keyboard,
 					const char          *rules,
