@@ -55,8 +55,6 @@
 #include <meta/types.h>
 #include <meta/main.h>
 #include "frame.h"
-#include "meta-idle-monitor-private.h"
-#include "monitor-private.h"
 
 #include "meta-surface-actor.h"
 #include "meta-surface-actor-wayland.h"
@@ -83,8 +81,8 @@ surface_set_buffer (MetaWaylandSurface *surface,
 
   if (surface->buffer)
     {
-      meta_wayland_buffer_unref (surface->buffer);
       wl_list_remove (&surface->buffer_destroy_listener.link);
+      meta_wayland_buffer_unref (surface->buffer);
     }
 
   surface->buffer = buffer;
